@@ -2,6 +2,7 @@ import os
 from os.path import dirname, join
 import multiprocessing as mp
 from typing import Union, List, Dict, Tuple
+import logging
 
 import pandas as pd
 import numpy as np
@@ -20,6 +21,10 @@ __all__ = [
     'plot_ranges',
 ]
 
+# Define logger.
+logger = logging.getLogger(__name__)
+logger.setLevel(logging.INFO)
+
 
 def split_list(l: list, n: int = 1) -> list:
     """Split list into n parts"""
@@ -97,7 +102,6 @@ def get_widgets(
         'red': [x for x in df.columns if 'red_' in x],
         'green': [x for x in df.columns if 'green_' in x],
         'blue': [x for x in df.columns if 'blue_' in x],
-        'gray': [x for x in df.columns if 'gray_' in x],
     }
     # Collect all options.
     col_options = {
@@ -109,7 +113,6 @@ def get_widgets(
         'red': [(x.split('_')[-1], x) for x in columns['red']],
         'green': [(x.split('_')[-1], x) for x in columns['green']],
         'blue': [(x.split('_')[-1], x) for x in columns['blue']],
-        'gray': [(x.split('_')[-1], x) for x in columns['gray']],
     }
     drop_options = [
         ('Hue', 'hue'),
@@ -118,7 +121,6 @@ def get_widgets(
         ('Red', 'red'),
         ('Green', 'green'),
         ('Blue', 'blue'),
-        ('Gray', 'gray'),
     ]
     # Prepare arguments for each plot group.
     args = {
@@ -190,9 +192,6 @@ def get_widgets(
             button_style='',
             icon=''
         )
-        # 'log_y': widgets.Checkbox(
-        #    value=False, description='Log scale',
-        # )
     }
     if group == 'channels':
         plot_wdgts['dropdown'] = widgets.Dropdown(
@@ -492,11 +491,12 @@ def plot_on_thumbnail(
             if occurrences >= min_tiles:
                 new_slides.append(slide)
         if len(new_slides) == 0:
-            print(f'No slides found with minimum of {min_tiles} tiles!')
+            logger.warning(
+                f'No slides found with minimum of {min_tiles} tiles!')
         slides = [x for x in slides if len(
             df[df.slide_name == x]) >= min_tiles]
     if len(slides) > max_thumbnails:
-        slides = np.random.choice(slides, max_thumbnails)
+        slides = np.random.choice(slides, max_thumbnails, replace=False)
     # Load thumbnails.
     thumbnails = {}
     for slide in slides:
@@ -518,7 +518,23 @@ def plot_on_thumbnail(
         annotated = ImageDraw.Draw(annotated_thumbnail)
         # Get tile coords.
         tmp = df[df.slide_name == slide]
-        coords = list(zip(tmp['x'], tmp['y'], tmp['width']))
+        width = tmp['width']
+        x = tmp['x']
+        y = tmp['y']
+        if 'spot_path' in tmp.columns:
+            # Try to load spot metadata.
+            data_path = os.path.dirname(tile_path.split('tiles')[0])
+            spot_meta_path = os.path.join(data_path, 'spot_metadata.csv')
+            try:
+                spot_meta = pd.read_csv(spot_meta_path)
+            except Exception as e:
+                raise IOError(f'Could not load {spot_meta_path}.')
+            # Merge with tmp so we can add spot coords to tile coords.
+            tmp = pd.merge(
+                tmp, spot_meta, left_on='spot_path', right_on='path')
+            x += tmp['x_y']
+            y += tmp['y_y']
+        coords = list(zip(x, y, width))
         # Draw.
         for x, y, width in coords:
             w_d = width/downsample
@@ -553,7 +569,7 @@ def plot_tiles(df: pd.DataFrame, nrows: int = 4, ncols: int = 16, px: int = 64)
             pd.DataFrame, type(df)
         ))
     if len(df) == 0:
-        print('No images.')
+        logger.warning('No images.')
         return
     df = df.sample(n=min(len(df), nrows*ncols))
     tiles = get_tiles(df, nrows=nrows, ncols=ncols, px=px, to_bytes=False)
